/*
 * Copyright (C) 2003 John David Ratliff
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;

import java.net.Socket;

/**
 * A class to handle client connections in a thread.
 */

public class Worker implements Runnable {
    private Socket socket;
    private byte[] file;
    private boolean single;

    /**
     * Create a new worker for a client connection.
     *
     * @param socket The client's socket.
     * @param file The data to send the client.
     * @param single true when sending one character at a time, false otherwise.
     */    
    protected Worker(Socket socket, byte[] file, boolean single) {
        this.socket = socket;
        this.file = file;
        this.single = single;
    }

    /**
     * The main work function. Send the data block to the client and close the socket.
     */    
    public void run() {
        InputStream in;
        OutputStream out;
        BufferedReader read;
        String message;
        
        try {
            // grab the I/O streams
            in = socket.getInputStream();
            out = socket.getOutputStream();
            read = new BufferedReader(new InputStreamReader(in));
            
            // wait for the client message
            while (in.available() < 1);
	    
            // read the client's message
            message = read.readLine();
            
            // print the signal message
            System.out.println("client send \"" + message + "\"");
            
            // send the file to the client
            if (single) {
                // write one char at a time...
                for (int i = 0; i < file.length; i++) {
                    out.write((int)file[i]);
                }
            } else {
                // write the buffer all at once...
                out.write(file);
            }
            
            // finally, close the client socket
            socket.close();  
        } catch (IOException e) {
            System.out.println("Unable to complete socket communication.");
            e.printStackTrace();
        }
    }
}
